#!/usr/bin/python
__author__ = 'gabrielet'

# Import to handle mininet topology
from mininet.net import Mininet
from mininet.node import RemoteController, OVSSwitch, UserSwitch
from mininet.log import setLogLevel
from mininet.cli import CLI


# Modify this class if you want to use another switch
class CustomSwitch(OVSSwitch):

    def __init__(self, name, **params):
        OVSSwitch.__init__(self, name=name, datapath='user', **params)


# This class creates topology for conf/simple
class TopologyGenerator(object):

    def __init__(self):
        # If you want to use a UserSwitch
        self.net = Mininet(controller=None, switch=CustomSwitch, listenPort=6634, inNamespace=False, autoSetMacs=True)
        # If you want to use a OVSSwitch
        # self.net = Mininet(controller=None, switch=CustomSwitch)
        # Add controller to the network
        self.net.addController('c0', controller=RemoteController, ip='127.0.0.1', port=6633)

    # Function to generate an hex representation of dpid
    @staticmethod
    def dpid(dpid):
        return hex(dpid)[2:]

    def create_topology(self):
        # Core Switches
        p11 = self.net.addSwitch("p11", dpid=self.dpid(11))
        p21 = self.net.addSwitch("p21", dpid=self.dpid(21))
        p31 = self.net.addSwitch("p31", dpid=self.dpid(31))

        p12 = self.net.addSwitch("p12", dpid=self.dpid(12))
        p22 = self.net.addSwitch("p22", dpid=self.dpid(22))
        p32 = self.net.addSwitch("p32", dpid=self.dpid(32))

        p13 = self.net.addSwitch("p13", dpid=self.dpid(13))
        p23 = self.net.addSwitch("p23", dpid=self.dpid(23))
        p33 = self.net.addSwitch("p33", dpid=self.dpid(33))

        p14 = self.net.addSwitch("p14", dpid=self.dpid(14))
        p24 = self.net.addSwitch("p24", dpid=self.dpid(24))
        p34 = self.net.addSwitch("p34", dpid=self.dpid(34))

        p15 = self.net.addSwitch("p15", dpid=self.dpid(15))
        p25 = self.net.addSwitch("p25", dpid=self.dpid(25))
        p35 = self.net.addSwitch("p35", dpid=self.dpid(35))

        p16 = self.net.addSwitch("p16", dpid=self.dpid(16))
        p26 = self.net.addSwitch("p26", dpid=self.dpid(26))
        p36 = self.net.addSwitch("p36", dpid=self.dpid(36))

        p17 = self.net.addSwitch("p17", dpid=self.dpid(17))
        p27 = self.net.addSwitch("p27", dpid=self.dpid(27))
        p37 = self.net.addSwitch("p37", dpid=self.dpid(37))

        p18 = self.net.addSwitch("p18", dpid=self.dpid(18))
        p28 = self.net.addSwitch("p28", dpid=self.dpid(28))
        p38 = self.net.addSwitch("p38", dpid=self.dpid(38))

        p19 = self.net.addSwitch("p19", dpid=self.dpid(19))
        p29 = self.net.addSwitch("p29", dpid=self.dpid(29))
        p39 = self.net.addSwitch("p39", dpid=self.dpid(39))

        p110 = self.net.addSwitch("p110", dpid=self.dpid(110))
        p210 = self.net.addSwitch("p210", dpid=self.dpid(210))
        p310 = self.net.addSwitch("p310", dpid=self.dpid(310))

        # PEs
        pe1 = self.net.addSwitch("pe1", dpid=self.dpid(4))
        pe2 = self.net.addSwitch("pe2", dpid=self.dpid(5))

        # Hosts
        h1 = self.net.addHost("h1")
        h2 = self.net.addHost("h2")

        # Hosts' links
        self.net.addLink(h1, pe1)
        self.net.addLink(h2, pe2)

        # Backbone links
        self.net.addLink(pe1, p11)
        self.net.addLink(pe1, p21)
        self.net.addLink(pe1, p31)

        self.net.addLink(p11, p12)
        self.net.addLink(p21, p22)
        self.net.addLink(p31, p32)

        self.net.addLink(p12, p13)
        self.net.addLink(p22, p23)
        self.net.addLink(p32, p33)

        self.net.addLink(p13, p14)
        self.net.addLink(p23, p24)
        self.net.addLink(p33, p34)

        self.net.addLink(p14, p15)
        self.net.addLink(p24, p25)
        self.net.addLink(p34, p35)

        self.net.addLink(p15, p16)
        self.net.addLink(p25, p26)
        self.net.addLink(p35, p36)

        self.net.addLink(p16, p17)
        self.net.addLink(p26, p27)
        self.net.addLink(p36, p37)

        self.net.addLink(p17, p18)
        self.net.addLink(p27, p28)
        self.net.addLink(p37, p38)

        self.net.addLink(p18, p19)
        self.net.addLink(p28, p29)
        self.net.addLink(p38, p39)

        self.net.addLink(p19, p110)
        self.net.addLink(p29, p210)
        self.net.addLink(p39, p310)

        self.net.addLink(p110, pe2)
        self.net.addLink(p210, pe2)
        self.net.addLink(p310, pe2)

    def configure_host(self):
        ### Host h1
        h1 = self.net.get("h1")
        h1.setIP("10.0.0.1/24")
        # Add default route
        h1.cmd("route add default h1-eth0")
        # Add static AR
        h1.cmd("arp -s 20.0.0.1 00:00:00:00:00:02")

        ### Host h2
        h2 = self.net.get("h2")
        h2.setIP("20.0.0.1/24")
        # Add default route
        h2.cmd("route add default h2-eth0")
        # Add static ARP
        h2.cmd("arp -s 10.0.0.1 00:00:00:00:00:01")

    def start_topology(self):
        # Start network
        self.net.start()
        # Configure hosts
        self.configure_host()
        # Open CLI
        CLI(self.net)
        self.net.stop()

if __name__ == "__main__":
    setLogLevel('info')
    t = TopologyGenerator()
    t.create_topology()
    t.start_topology()
