__author__ = 'gabrielet'

"""
Main class for VPN controller
"""

from ryu.base import app_manager

# OpenFlow specification 1.3
from ryu.ofproto import ofproto_v1_3

# To handle OpenFlow and controller events
from ryu.controller.handler import set_ev_cls

# To handle link event
from ryu.topology import event

# VPN system
from system import VpnSystem


class VpnController(app_manager.RyuApp):
    OFP_VERSIONS = [ofproto_v1_3.OFP_VERSION]

    # To observe network's events
    app_manager.require_app('ryu.topology.switches')

    def __init__(self, *args, **kwargs):
        super(VpnController, self).__init__(self, *args, **kwargs)

        # Start the VpnSystem
        self.system = VpnSystem()
        # Load configuration (included VPNs configuration) into the system
        self.system.load_system_configuration()
        # Initialize system
        self.system.init()

    @set_ev_cls(event.EventSwitchEnter)
    def switch_enter_handler(self, ev):
        # When a switch is added, then add it to the topology
        datapath = ev.switch.dp
        #print "#########", datapath.ports[1][0], datapath.ports[1][2]
        self.logger.info("Datapath %s entered", datapath.id)
        # Map interface name into port number
        self.system.map(datapath)
        # Add datapath to topology
        self.system.add_node(datapath)

    @set_ev_cls(event.EventSwitchLeave)
    def switch_leave_handler(self, ev):
        pass

    @set_ev_cls(event.EventLinkAdd)
    def link_add_handler(self, ev):
        link = ev.link
        self.logger.info('%s is up.', str(link))
        # Add link to topology
        self.system.add_link(link)

        # When a new link is added, try to search new paths in accord with VPN policies
        self.system.search()

    @set_ev_cls(event.EventLinkDelete)
    def link_delete_handler(self, ev):
        link = ev.link
        self.logger.info('%s is down.', str(link))

        # Remove link from topology
        self.system.delete_link(link)