__author__ = 'gabrielet'

"""
This class models nodes and edges to add to the graph
"""


class Node(object):
    # A Node wraps a model.vpn.Datapath object
    def __init__(self, dpid, datapath):
        self.dpid = dpid
        self.datapath = datapath

    def __hash__(self):
        return hash(self.dpid)

    def __repr__(self):
        return "Node(dpid=%s)" % self.dpid


class NeighborNode(object):
    def __init__(self, dpid, port):
        self.dpid = dpid
        self.outport = port

    def __repr__(self):
        return "NeighborNode(dpid=%s, port=%s)" % (self.dpid, self.outport)


class OutportGraph(object):
    def __init__(self):
        # The adjacency list of the nodes
        # Each entry is a pair in the form: <dpid, list_of_neighbors_with_outport>
        self.nodes = {}

    def add_node(self, node):
        # If node is not present in adjacency list, add it
        if node.dpid not in self.nodes.keys():
            self.nodes[node.dpid] = None

    # src and dst are Node objects
    def add_link(self, src, dst, src_to_port_no):
        # When a new link arrives, take node from adjacency list, create a NeighborNode
        # instance and add it in adjacency list
        neighbors = self.nodes.pop(src.dpid, None)
        # If neighbors is None, create the neighbors' list
        if neighbors is None:
            neighbors = []

        # Remember that a Ryu link object has in the form:
        # Link: Port<dpid=3, port_no=1, LIVE> to Port<dpid=1, port_no=2, LIVE>
        found = False
        for n in neighbors:
            if n.dpid == dst.dpid:
                found = True

        if not found:
            neighbor = NeighborNode(dst.dpid, src_to_port_no)
            neighbors.append(neighbor)

        # Update the nodes' map
        self.nodes[src.dpid] = neighbors

    # src and dst are Node objects
    def remove_edge(self, src, dst):
        # Take neighbors for src node
        src_neighbors = self.nodes.get(src.dpid, None)  # src_neighbors is a list(NeighborNode)
        if not src_neighbors is None:
            for src_neighbor in src_neighbors:
                if src_neighbor.dpid == dst.dpid:
                    src_neighbors.remove(src_neighbor)

    # This method return the outport of node src_node used to reach dst_node. src_node and dst_node are int (dpid)
    def get_outport(self, src_node, dst_node):
        neighbors = self.nodes.get(src_node, None)
        if not neighbors is None:
            for neighbor in neighbors:
                if neighbor.dpid == dst_node:
                    return neighbor.outport

    def info(self):
        for node in self.nodes.keys():
            print "Node", node, "reaches"
            neighbors = self.nodes.get(node)
            if neighbors is not None:
                for n in neighbors:
                    print n.dpid, "using port", n.outport